/**
 * @file
 *
 * Copyright (c) 2010 Aalto University) and RWTH Aachen University.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
/*
 * hipd oppdb.h
 *
 * Copyright (c) 2010 Aalto University) and RWTH Aachen University.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * Authors:
 * - Bing Zhou <bingzhou@cc.hut.fi>
 *
 */

#ifndef HIP_HIPD_OPPDB_H
#define HIP_HIPD_OPPDB_H

#include <stdint.h>
#include <netinet/in.h>

#include "lib/core/protodefs.h"
#include "lib/core/state.h"


struct hip_opp_blocking_request_entry {
    hip_hit_t           peer_phit;
    struct sockaddr_in6 caller;
    hip_hit_t           our_real_hit;

    time_t              creation_time;
    struct in6_addr     peer_ip;
    struct in6_addr     our_ip;
    uint8_t             proxy_flag; //0: normal connection, 1: connection through proxy
};

typedef struct hip_opp_blocking_request_entry hip_opp_block_t;

void hip_init_opp_db(void);
int hip_handle_opp_fallback(hip_opp_block_t *entry,
                            void *current_time);
hip_opp_block_t *hip_oppdb_find_byhits(const hip_hit_t *phit, struct sockaddr_in6 *src);
hip_opp_block_t *hip_oppdb_find_by_ip(const struct in6_addr *ip_peer);
hip_ha_t *hip_get_opp_hadb_entry(hip_hit_t *resp_hit,
                                 struct in6_addr *resp_addr);
int hip_oppdb_del_entry(const hip_hit_t *phit, const struct sockaddr_in6 *src);
void hip_oppdb_uninit(void);
int hip_oppdb_entry_clean_up(hip_opp_block_t *opp_entry);

hip_ha_t *hip_opp_add_map(const struct in6_addr *dst_ip,
                          const struct in6_addr *hit_our,
                          const struct sockaddr_in6 *caller);

hip_ha_t *hip_oppdb_get_hadb_entry_i1_r1(struct hip_common *msg,
                                         struct in6_addr *src_addr);
int hip_handle_opp_r1(struct hip_packet_context *ctx);
int hip_for_each_opp(int (*func)(hip_opp_block_t *entry, void *opaq),
                     void *opaque);
int hip_opp_get_peer_hit(struct hip_common *msg,
                         const struct sockaddr_in6 *src);

#endif /* HIP_HIPD_OPPDB_H */
